import 'dotenv/config';
import fetch from 'node-fetch';

import admin, { ServiceAccount } from 'firebase-admin';
import { FieldValue } from 'firebase-admin/firestore';

import serviceAccount from './google-services.json' assert { type: "json" };

const RAIN_GAUGE_ENDPOINT = process.env.RAIN_GAUGE_ENDPOINT;
const SOIL_SENSOR_ENDPOINT = process.env.SOIL_SENSOR_ENDPOINT;
const TEASENSE_ENDPOINT = process.env.TEASENSE_ENDPOINT;
const DATA_COLLECT_SLEEP = 2000;

type DataPoint = {
  name: string;
  value: number;
  timestamp: Date | null;
};

const app = admin.initializeApp({
  credential: admin.credential.cert(serviceAccount as ServiceAccount),
  databaseURL: process.env.DATABASE_URL ?? '',
});

const db = app.firestore();

const saveDataPoint = async (name: string, value: number, noTimestamp?: boolean) => {
  const docRef = db.collection('colligit').doc(name);

  const values: DataPoint[] = [
    {
      name,
      timestamp: (noTimestamp) ? null : new Date(),
      value,
    }
  ];

  if (!noTimestamp) {
    await docRef.set({
      data: FieldValue.arrayUnion(...values),
    }, { merge: true });
  } else {
    await docRef.set({
      data: FieldValue.arrayUnion(...values),
    });
  }
}

const collectData = async () => {
  if (RAIN_GAUGE_ENDPOINT && RAIN_GAUGE_ENDPOINT.trim() !== '') {
    try {
      const response = await fetch(RAIN_GAUGE_ENDPOINT, {
        signal: AbortSignal.timeout(5000),
      });
      const data: any = await response.json();
      if (data.rainfall > 0) {
        await saveDataPoint('rainfall', data.rainfall);
      }
    } catch (e) {
      console.error(e);
    }
  }

  if (SOIL_SENSOR_ENDPOINT && SOIL_SENSOR_ENDPOINT.trim() !== '') {
    try {
      const response = await fetch(SOIL_SENSOR_ENDPOINT, {
        signal: AbortSignal.timeout(5000),
      });
      const data: any = await response.json();
      await saveDataPoint('soil_voltage', data.moisture);
    } catch (e) {
      console.error(e);
    }
  }

  if (TEASENSE_ENDPOINT && TEASENSE_ENDPOINT.trim() !== '') {
    try {
      const response = await fetch(TEASENSE_ENDPOINT, {
        signal: AbortSignal.timeout(10000),
      });
      const data: any = await response.json();
      await saveDataPoint('teasense', data.temperature, true);
    } catch (e) {
      console.error(e);
    }
  }

  setTimeout(() => collectData(), DATA_COLLECT_SLEEP);
};

setTimeout(() => collectData(), DATA_COLLECT_SLEEP);
